"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.StellateService = void 0;
const tslib_1 = require("tslib");
const core_1 = require("@vendure/core");
const common_1 = require("@nestjs/common");
const node_fetch_1 = (0, tslib_1.__importDefault)(require("node-fetch"));
const constants_1 = require("../constants");
let StellateService = class StellateService {
    constructor(options, connection, entityHydrator) {
        this.options = options;
        this.connection = connection;
        this.entityHydrator = entityHydrator;
        this.purgeApiUrl = `https://admin.stellate.co/${options.serviceName}`;
    }
    purgeProducts(products) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            core_1.Logger.verbose(`Purging cache: Product(${products.map((p) => p.id).join(', ')})`, constants_1.loggerCtx);
            yield this.purge('Product', products.map((p) => p.id));
        });
    }
    purgeProductVariants(productVariants) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            core_1.Logger.verbose(`Purging cache: ProductVariant(${productVariants.map((p) => p.id).join(', ')})`, constants_1.loggerCtx);
            yield this.purge('ProductVariant', productVariants.map((p) => p.id));
        });
    }
    purgeSearchResults(items) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            const productIds = items.map((item) => (item instanceof core_1.Product ? item.id : item.productId));
            core_1.Logger.verbose(`Purging cache: SearchResult(${productIds.join(', ')})`, constants_1.loggerCtx);
            yield this.purge('SearchResult', productIds, 'productId');
        });
    }
    purgeAllOfType(type) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            core_1.Logger.verbose(`Purging cache: All ${type}s`, constants_1.loggerCtx);
            yield this.purge(type);
        });
    }
    purgeCollections(collections) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            core_1.Logger.verbose(`Purging cache: Collection(${collections.map((c) => c.id).join(', ')}})`, constants_1.loggerCtx);
            yield this.purge('Collection', collections.map((p) => p.id));
        });
    }
    purgeSearchResponseCacheIdentifiers(collections) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            const slugs = collections.map((c) => { var _a, _b, _c; return (_a = c.slug) !== null && _a !== void 0 ? _a : (_c = (_b = c.translations) === null || _b === void 0 ? void 0 : _b[0]) === null || _c === void 0 ? void 0 : _c.slug; });
            if (slugs.length) {
                core_1.Logger.verbose(`Purging cache: SearchResponseCacheIdentifier(${slugs.join(', ')})`, constants_1.loggerCtx);
                yield this.purge('SearchResponseCacheIdentifier', slugs);
            }
        });
    }
    purge(type, keys, keyName = 'id') {
        const payload = {
            query: `
                mutation PurgeType($type: String!, $keyFields: [KeyFieldInput!]) {
                    _purgeType(type: $type, keyFields: $keyFields)
                }
            `,
            variables: {
                type,
                keyFields: keys === null || keys === void 0 ? void 0 : keys.filter((id) => !!id).map((id) => ({ name: keyName, value: id.toString() })),
            },
        };
        if (this.options.debugMode === true) {
            if (this.options.debugLogging === true) {
                const keyFieldsLength = payload.variables.keyFields.length;
                if (5 < keyFieldsLength) {
                    payload.variables.keyFields = payload.variables.keyFields.slice(0, 5);
                }
                core_1.Logger.debug('DEBUG MODE: Purge arguments:\n' + JSON.stringify(payload.variables, null, 2), constants_1.loggerCtx);
                if (5 < keyFieldsLength) {
                    core_1.Logger.debug(`(A further ${keyFieldsLength - 5} keyFields truncated)`, constants_1.loggerCtx);
                }
            }
        }
        else {
            return (0, node_fetch_1.default)(this.purgeApiUrl, {
                method: 'POST',
                headers: {
                    'Content-Type': 'application/json',
                    'stellate-token': this.options.apiToken,
                },
                body: JSON.stringify(payload),
                timeout: 5000,
            })
                .then((res) => res.json())
                .then((json) => {
                var _a, _b, _c;
                if (((_a = json.data) === null || _a === void 0 ? void 0 : _a._purgeType) !== true) {
                    core_1.Logger.error(`Purge failed: ${(_c = (_b = json.errors) === null || _b === void 0 ? void 0 : _b.map((e) => e.message).join(', ')) !== null && _c !== void 0 ? _c : JSON.stringify(json)}`, constants_1.loggerCtx);
                }
            })
                .catch((err) => {
                core_1.Logger.error(`Purge error: ${err.message}`, constants_1.loggerCtx);
            });
        }
    }
};
StellateService = (0, tslib_1.__decorate)([
    (0, tslib_1.__param)(0, (0, common_1.Inject)(constants_1.STELLATE_PLUGIN_OPTIONS)),
    (0, tslib_1.__metadata)("design:paramtypes", [Object, core_1.TransactionalConnection,
        core_1.EntityHydrator])
], StellateService);
exports.StellateService = StellateService;
//# sourceMappingURL=stellate.service.js.map